#import "osd/osd.hpp"

t_option option;

extern int __argc;
extern char **__argv;

void do_config(char *file)
{
    /* Our token list */
    int i, argc;
    char *argv[TOKEN_LIST_SIZE];

    set_option_defaults();
    for(i = 0; i < TOKEN_LIST_SIZE; i += 1) argv[i] = NULL;

    /* Check configuration file */
    if(file) parse_file(file, &argc, argv);

    /* Check extracted tokens */
    parse_args(argc, argv);

    /* Free token list */
    for(i = 0; i < argc; i += 1) if(argv[argc]) free (argv[argc]);

    /* Check command line */
    parse_args(__argc, __argv);
}


/* Parse configuration file */
int parse_file(char *filename, int *argc, char **argv)
{
    char token[0x100];
    FILE *handle = NULL;

    *argc = 0;
    handle = fopen(filename, "r");
    if(!handle) return (0);

    fscanf(handle, "%s", &token[0]);
    while(!(feof(handle)))
    {
        int size = strlen(token) + 1;
        argv[*argc] = (char *)malloc(size);
        if(!argv[*argc]) return (0);
        strcpy(argv[*argc], token);
        *argc += 1;
        fscanf(handle, "%s", &token[0]);
    }

    if(handle) fclose(handle);
    return (1);
}


int check_bool(char *token)
{
    int result = 1;
    if(stricmp("off", token) == 0)
        result = 0;
    if(stricmp("no", token) == 0)
        result = 0;
    return (result);
}


void set_option_defaults(void)
{
#ifdef ALLEGRO_DOS
    option.video_driver = GFX_AUTODETECT;
#else
    option.video_driver = GFX_AUTODETECT_WINDOWED;
#endif

    option.video_width  = 640;
    option.video_height = 480;
    option.video_depth  = -1;
    option.video_refresh = 60;
    option.video_filter = VIDEO_FILTER_FIT;

    option.fixrgb        = -1;
    option.rtpal         = 0;
    option.vsync         = 1;
    option.throttle      = 1;
    option.skip          = -1;
    option.showfps       = 0;
    option.scanlines     = 0;

    option.sound        = 1;
    option.sndcard      = DIGI_AUTODETECT;
    option.sndrate      = -1;
    option.sndinterp    = 0;
    option.sndvol       = 1.0f;
    option.sound_filter = SOUND_FILTER_NONE;
    option.swap         = 0;

    option.nopsg        = 0;
    option.nofm         = 0;
    option.uberpsg      = 0;

    option.joy_driver   = JOY_TYPE_AUTODETECT;
}


void print_options(void)
{
    int j;

    printf ("General options:\n");
    printf("    -sys.throttle <on|off> Enable speed throttling (default=on)\n");
    printf("    -sys.skip <n>          Specify frame skip level (default=auto)\n");
    printf ("\n");
    printf ("Video options:\n");
    printf("    -vid.driver <s>        Select video driver (default=auto window)\n");
    printf("    -vid.res <w> <h>       Specify display resolution (default=640x480)\n");
    printf("    -vid.depth <n>         Specify display depth (default=auto)\n");
    printf("    -vid.refresh <n>       Specify a prefered refresh rate (default=60)\n");
    printf("    -vid.filter <s>        Set video filter (default=fit)\n");
    printf("    -vid.scanlines <n>     Set scanlines intensity (0-100, default=off)\n");
    printf("    -vid.vsync <on|off>    Enable vsync polling (default=on)\n");
    printf("    -vid.swap <on|off>     Enable RGB<>BGR swapping (default=auto)\n");
    printf ("\n");
    printf ("Video drivers:\n");
    printf ("    ");
      for(j = 0; video_driver_table[j].token != NULL; j += 1)
          printf ("%s ", video_driver_table[j].token);
    printf ("\n");
    printf ("\n");
    printf ("Video filters:\n");
    printf ("    ");
      for(j = 0; video_filter_table[j].token != NULL; j += 1)
          printf ("%s ", video_filter_table[j].token);
    printf ("\n");
    printf ("\n");
    printf ("Sound options:\n");
    printf("    -snd.output <on|off>   Enable sound output (default=on)\n");
    printf("    -snd.driver <s>        Select sound driver (default=auto)\n");
    printf("    -snd.rate <n>          Select sampling rate (default=auto)\n");
    printf("    -snd.interp <on|off>   Enable sound interpolation (default=off)\n");
    printf("    -snd.volume <n>        Specify sound volume level (1.0=100%%, default=1.0)\n");
    printf("    -snd.filter <s>        Set sound filter (default=none)\n");
    printf("    -snd.swap <on|off>     Swap left and right channels (default=off)\n");
    printf("    -psg.stereo <on|off>   Enable stereo PSG output (default=off)\n");
    printf ("\n");
    printf ("Sound drivers:\n");
    printf ("    ");
      for(j = 0; sound_driver_table[j].token != NULL; j += 1)
          printf ("%s ", sound_driver_table[j].token);
    printf ("\n");
    printf ("\n");
    printf ("Sound filters:\n");
    printf ("    ");
      for(j = 0; sound_filter_table[j].token != NULL; j += 1)
          printf ("%s ", sound_filter_table[j].token);
    printf ("\n");
    printf ("\n");
    printf ("Input options:\n");
    printf("    -joy.driver <s>        Select joystick driver (default=auto)\n");
    printf ("\n");
    printf ("Joystick drivers:\n");
    printf ("    ");
      for(j = 0; joy_driver_table[j].token != NULL; j += 1)
          printf ("%s ", joy_driver_table[j].token);
    printf ("\n");
}


void parse_args(int argc, char **argv)
{
    int i, j;

    for(i = 0; i < argc; i += 1)
    {
        if(stricmp("-sys.throttle", argv[i]) == 0)
        {
            option.throttle = check_bool(argv[i+1]);
        }

        if(stricmp("-sys.skip", argv[i]) == 0)
        {
            option.skip = atoi(argv[i+1]);
            if(!option.skip) option.skip = -1;
        }

        if(stricmp("-vid.driver", argv[i]) == 0)
        {
            for(j = 0; video_driver_table[j].token != NULL; j += 1)
            {
                if(stricmp(argv[i+1], video_driver_table[j].token) == 0)
                    option.video_driver = video_driver_table[j].value;
            }
        }

        if(stricmp("-vid.res", argv[i]) == 0)
        {
            option.video_width = atoi(argv[i+1]);
            option.video_height = atoi(argv[i+2]);
        }

        if(stricmp("-vid.depth", argv[i]) == 0)
        {
            option.video_depth = atoi(argv[i+1]);
        }

        if(stricmp("-vid.refresh", argv[i]) == 0)
        {
            option.video_refresh = atoi(argv[i+1]);
        }

        if(stricmp("-vid.filter", argv[i]) == 0)
        {
            for(j = 0; video_filter_table[j].token != NULL; j += 1)
            {
                if(stricmp(argv[i+1], video_filter_table[j].token) == 0)
                    option.video_filter = video_filter_table[j].value;
            }
        }

        if(stricmp("-vid.scanlines", argv[i]) == 0)
        {
            option.scanlines = atoi(argv[i+1]);
        }

        if(stricmp("-vid.vsync", argv[i]) == 0)
        {
            option.vsync = check_bool(argv[i+1]);
        }

        if(stricmp("-vid.swap", argv[i]) == 0)
        {
            option.fixrgb = check_bool(argv[i+1]);
        }

        if(stricmp("-snd.output", argv[i]) == 0)
        {
            option.sound = check_bool(argv[i+1]);
        }

        if(stricmp("-snd.driver", argv[i]) == 0)
        {
            for(j = 0; sound_driver_table[j].token != NULL; j += 1)
            {
                if(stricmp(argv[i+1], sound_driver_table[j].token) == 0)
                    option.sndcard = sound_driver_table[j].value;
            }
        }

        if(stricmp("-snd.rate", argv[i]) == 0)
        {
            option.sndrate = atoi(argv[i+1]);
        }

        if(stricmp("-snd.interp", argv[i]) == 0)
        {
            option.sndinterp = check_bool(argv[i+1]);
        }

        if(stricmp("-snd.volume", argv[i]) == 0)
        {
            option.sndvol = atof(argv[i+1]);
        }

        if(stricmp("-snd.filter", argv[i]) == 0)
        {
            for(j = 0; sound_filter_table[j].token != NULL; j += 1)
            {
                if(stricmp(argv[i+1], sound_filter_table[j].token) == 0)
                    option.sound_filter = sound_filter_table[j].value;
            }
        }

        if(stricmp("-snd.swap", argv[i]) == 0)
        {
            option.swap = check_bool(argv[i+1]);
        }

        if(stricmp("-psg.stereo", argv[i]) == 0)
        {
            option.uberpsg = check_bool(argv[i+1]);
        }

        if(stricmp("-joy.driver", argv[i]) == 0)
        {
            for(j = 0; joy_driver_table[j].token != NULL; j += 1)
            {
                if(stricmp(argv[i+1], joy_driver_table[j].token) == 0)
                    option.joy_driver = joy_driver_table[j].value;
            }
        }
    }
}
