#import "osd/osd.hpp"

#define NUM_PLAYERS  2

// VM side

enum
{
   INPUT_BUTTON_MODE = 0,
   INPUT_BUTTON_Z,
   INPUT_BUTTON_Y,
   INPUT_BUTTON_X,
   INPUT_BUTTON_START,
   INPUT_BUTTON_C,
   INPUT_BUTTON_B,
   INPUT_BUTTON_A,
   INPUT_BUTTON_LEFT,
   INPUT_BUTTON_RIGHT,
   INPUT_BUTTON_DOWN,
   INPUT_BUTTON_UP,
   INPUT_BUTTONS
};

typedef struct _INPUT_BUTTON
{
   const char *name;
   int id;

} INPUT_BUTTON;

static INPUT_BUTTON buttons[INPUT_BUTTONS] =
{
   { "Left",  INPUT_BUTTON_LEFT  },
   { "Right", INPUT_BUTTON_RIGHT },
   { "Up",    INPUT_BUTTON_UP    },
   { "Down",  INPUT_BUTTON_DOWN  },
   { "A",     INPUT_BUTTON_A     },
   { "B",     INPUT_BUTTON_B     },
   { "C",     INPUT_BUTTON_C     },
   { "X",     INPUT_BUTTON_X     },
   { "Y",     INPUT_BUTTON_Y     },
   { "Z",     INPUT_BUTTON_Z     },
   { "Start", INPUT_BUTTON_START },
   { "Mode",  INPUT_BUTTON_MODE  }
};

// Host side

#define MAX_INPUTS   2

enum
{
   INPUT_DEVICE_TYPE_NONE,
   INPUT_DEVICE_TYPE_KEYBOARD,
   INPUT_DEVICE_TYPE_JOYSTICK
};

typedef struct _INPUT_DEVICE
{
} INPUT_DEVICE;

typedef struct _INPUT_MAP
{
   INPUT_DEVICE inputs[MAX_INPUTS];

} INPUT_MAP;

/* Check if a key is pressed */
static INLINE int check_key(int code)
{
    static char lastbuf[0x100] = {0};

    if((!key[code]) && (lastbuf[code] == 1))
        lastbuf[code] = 0;

    if((key[code]) && (lastbuf[code] == 0))
    {
        lastbuf[code] = 1;
        return (1);
    }                                                                    

    return (0);
}


void osd_input_update (void)
{
    if(key[KEY_ESC] || key[KEY_END])
    {
        quit = 1;
    }

    input.pad[0] = 0;

    /* Is the joystick being used ? */
    if(option.joy_driver != JOY_TYPE_NONE)
    {
        poll_joystick();

        /* Check player 1 joystick */
        if(joy[0].stick[0].axis[1].d1) input.pad[0] |= INPUT_UP;
        else
        if(joy[0].stick[0].axis[1].d2) input.pad[0] |= INPUT_DOWN;

        if(joy[0].stick[0].axis[0].d1) input.pad[0] |= INPUT_LEFT;
        else
        if(joy[0].stick[0].axis[0].d2) input.pad[0] |= INPUT_RIGHT;

        if(joy[0].button[0].b)  input.pad[0] |= INPUT_A;
        if(joy[0].button[1].b)  input.pad[0] |= INPUT_B;
        if(joy[0].button[2].b)  input.pad[0] |= INPUT_C;
        if(joy[0].button[3].b)  input.pad[0] |= INPUT_START;
        if(joy[0].button[4].b)  input.pad[0] |= INPUT_X;
        if(joy[0].button[5].b)  input.pad[0] |= INPUT_Y;
        if(joy[0].button[6].b)  input.pad[0] |= INPUT_Z;
        if(joy[0].button[7].b)  input.pad[0] |= INPUT_MODE;

        /* More than one joystick supported ? */
        if(num_joysticks > 2)
        {
            /* Check player 2 joystick */
            if(joy[1].stick[0].axis[1].d1) input.pad[1] |= INPUT_UP;
            else
            if(joy[1].stick[0].axis[1].d2) input.pad[1] |= INPUT_DOWN;

            if(joy[1].stick[0].axis[0].d1) input.pad[1] |= INPUT_LEFT;
            else
            if(joy[1].stick[0].axis[0].d1) input.pad[1] |= INPUT_RIGHT;

            if(joy[1].button[0].b)  input.pad[1] |= INPUT_A;
            if(joy[1].button[1].b)  input.pad[1] |= INPUT_B;
            if(joy[1].button[2].b)  input.pad[1] |= INPUT_C;
            if(joy[1].button[3].b)  input.pad[1] |= INPUT_START;
            if(joy[1].button[4].b)  input.pad[1] |= INPUT_X;
            if(joy[1].button[5].b)  input.pad[1] |= INPUT_Y;
            if(joy[1].button[6].b)  input.pad[1] |= INPUT_Z;
            if(joy[1].button[7].b)  input.pad[1] |= INPUT_MODE;
        }
    }

    if(key[KEY_UP])     input.pad[0] |= INPUT_UP;
    else
    if(key[KEY_DOWN])   input.pad[0] |= INPUT_DOWN;
    if(key[KEY_LEFT])   input.pad[0] |= INPUT_LEFT;
    else
    if(key[KEY_RIGHT])  input.pad[0] |= INPUT_RIGHT;

    if(key[KEY_A])      input.pad[0] |= INPUT_A;
    if(key[KEY_S])      input.pad[0] |= INPUT_B;
    if(key[KEY_D])      input.pad[0] |= INPUT_C;
    if(key[KEY_F])      input.pad[0] |= INPUT_START;
    if(key[KEY_Z])      input.pad[0] |= INPUT_X;
    if(key[KEY_X])      input.pad[0] |= INPUT_Y;
    if(key[KEY_C])      input.pad[0] |= INPUT_Z;
    if(key[KEY_V])      input.pad[0] |= INPUT_MODE;

    if (check_key(KEY_F1))
    {
      char path[MAX_FILENAMEZ];
      int depth;
      BITMAP *tmp;

      memset (path, 0, sizeof (path));
      replace_extension (path, get_filename (__argv[1]), empty_string, (sizeof (path) - 1));

      depth = bitmap_color_depth (screen);
      if (depth == 32)
         depth = 24;

      tmp = create_bitmap_ex (depth, SCREEN_W, SCREEN_H);
      if (tmp)
      {
         int index;
         PALETTE pal;

         blit (screen, tmp, 0, 0, 0, 0, SCREEN_W, SCREEN_H);

         get_palette (pal);

         for (index = 0; index < 999; index++)
         {
            char filename[MAX_FILENAMEZ];

            sprintf (filename, "%s_%03d.png", path, index);
            if (exists (filename))
               continue;

            save_png (filename, tmp, pal);
            break;
         }

         destroy_bitmap (tmp);
      }
    }

    if (check_key(KEY_F3))
      option.throttle = !option.throttle;
    if (check_key(KEY_F4))
      option.vsync = !option.vsync;

    if (check_key(KEY_F5))
      option.showfps = !option.showfps;

    if (check_key(KEY_F7))
      option.nopsg = !option.nopsg;
    if (check_key(KEY_F8))
      option.nofm = !option.nofm;

    if (check_key(KEY_F9))
    {
      snd.volume -= 0.20f;
      if (snd.volume < 0)
         snd.volume = 0;
    }

    if (check_key(KEY_F10))
      snd.volume += 0.20f;

    if(check_key(KEY_F12))
      reset = TRUE;
}   
