#import "osd/osd.hpp"

#define BUFFER_SIZE  (option.sndrate / FRAMES_PER_SECOND)

static AUDIOSTREAM *stream = NULL;

int osd_sound_init (void)
{
   int rate;

   set_volume_per_voice (0);
   if (install_sound (option.sndcard, MIDI_NONE, NULL) != 0)
      return (1);

   if (option.sndrate == -1)
      option.sndrate = get_mixer_frequency ();

   if (option.sndinterp)
   {
      option.sndrate *= 2;
      set_mixer_quality (2);
   }
   else
   {
      set_mixer_quality (1);
   }
   if (digi_driver->id == DIGI_NONE)
      return (0);

   stream = play_audio_stream (BUFFER_SIZE, 16, TRUE, option.sndrate, 255, 128);
   if (!stream)
      return (3);

   return (0);
}

void osd_sound_shutdown (void)
{
   if (stream)
   {
      stop_audio_stream (stream);
      stream = NULL;
   }

   remove_sound ();
}

void osd_sound_pause (void)
{
   ASSERT(stream);

   voice_stop (stream->voice);
}

void osd_sound_resume (void)
{
   ASSERT(stream);

   voice_start (stream->voice);
}

void osd_sound_buffer (signed short *ldata, signed short *rdata, unsigned size)
{
   unsigned short *buffer;
   unsigned index;

   ASSERT(stream);
   ASSERT(ldata);
   ASSERT(rdata);

   if (digi_driver->id == DIGI_NONE)
      return;

   buffer = (unsigned short *)get_audio_stream_buffer (stream);
   if (!buffer)
      return;

   if (option.swap)
   {
      /* Stereo channel swap. */

      signed short *old_ldata;

      old_ldata = ldata;
      ldata = rdata;
      rdata = old_ldata;
   }

   for (index = 0; index < (size / 2); index++)
   {          
      int16 lsamp, rsamp;

      /* Copy mono images to the stereo image, converting from signed to
         unsigned as needed. */

      lsamp = ldata[index];
      rsamp = rdata[index];

      switch (option.sound_filter)
      {
         case SOUND_FILTER_LOWPASS:
         {
            static int16 last_lsamp = 0, last_rsamp = 0;

            lsamp = (lsamp + last_lsamp) >> 1;
            rsamp = (rsamp + last_rsamp) >> 1;

            last_lsamp = lsamp;
            last_rsamp = rsamp;

            break;
         }

         default:
            break;
      }

      buffer[((index * 2) + 0)] = (lsamp ^ 0x8000);
      buffer[((index * 2) + 1)] = (rsamp ^ 0x8000);
   }

   free_audio_stream_buffer (stream);
}                                           
